 /** ==========================================================================
 * 
 * @file		SoundPercept.h --
 * @brief		Header file for class representing a single sound percept.
 *
 * @author	Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 * 
 * ========================================================================== 
 **/

#ifndef __SOUNDPERCEPT_H__
#define __SOUNDPERCEPT_H__

#include "btl/Type.h"
#include "btl/Percept.h"

#include <xmltio/xmltio.hpp>
#include <fstream>
#include <string>

namespace airobots {

class SoundPercept : public Percept {

	public:

		static std::string getBaseTag();
		static std::string getTemplate();
		static std::string getPerceptName();

		/** Constructor. */
		SoundPercept(int id);

		/** Destructor. */
		~SoundPercept();

		/**
		 * Returns the information of this sound percept in an xml string. 
		 * (This function is deprecated.)
		 */
		std::string toXml();

		/** Adds a new spectrum to this sound percept. */
		void addSpectrum(std::vector<int> spectrum);

		/** Sets a new label to this percept. */
		void setLabel(std::string label);
		
		/** Returns the current label of the percept. */
		std::string getLabel();

		/** Get current maximal frequency bands. */
		void getCurrentMaxBands(std::vector<int> &values);
	
		/** Stores a vector of location estimations in this percept. */
		void setLocationEstimation(std::vector<float> angles);

		/** Writes this percept to the assigned output stream. */
		void writeToStream(std::fstream *outputstream);

		/** Fills the data of the sound percept into an xmltio::Location. */
		void fillInto(xmltio::Location &loc) const;

		void sanitizeLocation(xmltio::Location &loc) const;

		/** Extracts a sound percept from an xmltio::Location. */
		static void fromLocation(const xmltio::Location &loc, SoundPercept &percept);		

	private:

		// ======================================================================
		// Private fields.
		// ======================================================================

		/** The tracking id of the sound percept. Should be removed later. */
		int id;

		/** The label for the sound. */
		std::string label;
		
		/** A counter how many spectras where assigned to this percept. */
		int spectraCounter;

		/** Average most important frequency bands. */
		std::vector<float> averages;

		/** A vector of location estimations. */
		std::vector<float> angles;

		// ======================================================================
		// Private methods. 
		// ======================================================================

		/** Computes an incremental mean value. */
		float incrementalMean(float oldValue, float newValue, float addedValues);

};

}

#endif /** __SOUNDPERCEPT_H__ */

