#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/DynamicGridMap.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(DynamicGridMapTest)

BOOST_AUTO_TEST_CASE(testMemoryIntegrity) {

	DynamicGridMap original;
	original.setGenerator("test");
	original.setTimestamp(Timestamp(123, 456));
	original.setOriginX(123);
	original.setOriginY(456);
	original.setResolution(0.23);
	original.setSizeX(34234);
	original.setSizeY(2345623);
	original.setUri("blubb/bla");

	vector<float> gridMap;
	gridMap.push_back(0.0f);
	gridMap.push_back(1.0f);
	gridMap.push_back(2.0f);
	gridMap.push_back(3.0f);

	original.setGridMap(gridMap, 2, 2);

	// copy
	DynamicGridMap copy(original);

	BOOST_REQUIRE_CLOSE(copy.getGridMap()(0,0), 0.0f, tolerance);
	BOOST_REQUIRE_CLOSE(copy.getGridMap()(0,1), 1.0f, tolerance);
	BOOST_REQUIRE_CLOSE(copy.getGridMap()(1,0), 2.0f, tolerance);
	BOOST_REQUIRE_CLOSE(copy.getGridMap()(1,1), 3.0f, tolerance);

	// assign
	copy = original;

	BOOST_REQUIRE_CLOSE(copy.getGridMap()(0,0), 0.0f, tolerance);
	BOOST_REQUIRE_CLOSE(copy.getGridMap()(0,1), 1.0f, tolerance);
	BOOST_REQUIRE_CLOSE(copy.getGridMap()(1,0), 2.0f, tolerance);
	BOOST_REQUIRE_CLOSE(copy.getGridMap()(1,1), 3.0f, tolerance);

	// pointer
	DynamicGridMapPtr ptr(new DynamicGridMap(original));

	BOOST_REQUIRE_CLOSE(ptr->getGridMap()(0,0), 0.0f, tolerance);
	BOOST_REQUIRE_CLOSE(ptr->getGridMap()(0,1), 1.0f, tolerance);
	BOOST_REQUIRE_CLOSE(ptr->getGridMap()(1,0), 2.0f, tolerance);
	BOOST_REQUIRE_CLOSE(ptr->getGridMap()(1,1), 3.0f, tolerance);
}

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		DynamicGridMap original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setOriginX(123);
		original.setOriginY(456);
		original.setResolution(0.23);
		original.setSizeX(34234);
		original.setSizeY(2345623);
		original.setUri("blubb/bla");

		Location originalLoc(DynamicGridMap::getTemplate(), DynamicGridMap::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true);

		DynamicGridMap parsed = extract<DynamicGridMap>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getOriginX(), original.getOriginX());
		BOOST_REQUIRE_EQUAL(parsed.getOriginY(), original.getOriginY());
		BOOST_REQUIRE_CLOSE(parsed.getResolution(), original.getResolution(), tolerance);
		BOOST_REQUIRE_EQUAL(parsed.getSizeX(), original.getSizeX());
		BOOST_REQUIRE_EQUAL(parsed.getSizeY(), original.getSizeY());
		BOOST_REQUIRE_EQUAL(parsed.getUri(), original.getUri());

	} catch (xmltio::XMLTIO_Exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("DynamicGridMap.xml"), DynamicGridMap::getBaseTag());
	DynamicGridMap parsed = extract<DynamicGridMap>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(74, parsed.getOriginX());
	BOOST_REQUIRE_EQUAL(34, parsed.getOriginY());
	BOOST_REQUIRE_CLOSE((float) 0.45, parsed.getResolution(), tolerance);
	BOOST_REQUIRE_EQUAL(745, parsed.getSizeX());
	BOOST_REQUIRE_EQUAL(123, parsed.getSizeY());
	BOOST_REQUIRE_EQUAL("test/uri", parsed.getUri());

}

BOOST_AUTO_TEST_SUITE_END()
