/*
 * PlaneDataTest.cpp
 *
 *  Created on: Jul 21, 2010
 *      Author: lziegler
 */

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include <sys/time.h>
#include <time.h>

#include "btl/PlaneData.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double TOLERANCE = 0.0001;

BOOST_AUTO_TEST_SUITE(PlaneDataTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		vector<pair<double,double> > polygon;
		polygon.push_back(pair<double,double>(0,0));
		polygon.push_back(pair<double,double>(1,1));
		polygon.push_back(pair<double,double>(2,2));

		PlaneData original;
		original.setScope(PlaneData::LOCAL);
		original.setPolygon(polygon, MILLIMETER);

		original.setGenerator("generator123");
		original.setTimestamp(Timestamp());

		original.setTranslation(1.0,1.0,1.0,MILLIMETER);
		original.setNormalRotation(2.0,2.0,2.0,RADIANT);

		double milli = original.getPolygon(MILLIMETER)[2].first;
		double centi = original.getPolygon(CENTIMETER)[2].first;
		double meter = original.getPolygon(METER)[2].first;

		BOOST_REQUIRE_CLOSE(milli, centi * 10, TOLERANCE);
		BOOST_REQUIRE_CLOSE(milli, meter * 1000, TOLERANCE);

		original.setPolygon(polygon, MILLIMETER);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, 2.0, TOLERANCE);
		original.setPolygon(polygon, METER);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, 2000.0, TOLERANCE);
		original.setPolygon(polygon, CENTIMETER);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, 20.0, TOLERANCE);
		original.setPolygon(polygon, MILLIMETER);

		Location originalLoc(PlaneData::getTemplate(), PlaneData::getBaseTag());

		originalLoc = original;

		cout << originalLoc.getDocumentText(true) << endl;

		PlaneData parsed = extract<PlaneData>(originalLoc);

		BOOST_REQUIRE_EQUAL(original.getGenerator(), parsed.getGenerator());
		BOOST_REQUIRE_EQUAL(original.getTimestamp(), parsed.getTimestamp());
		BOOST_REQUIRE_EQUAL(original.getScope(), parsed.getScope());
		BOOST_REQUIRE_CLOSE(original.getTranslationX(MILLIMETER), parsed.getTranslationX(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getTranslationY(MILLIMETER), parsed.getTranslationY(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getTranslationZ(MILLIMETER), parsed.getTranslationZ(MILLIMETER), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getNormalRotationX(RADIANT), parsed.getNormalRotationX(RADIANT), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getNormalRotationY(RADIANT), parsed.getNormalRotationY(RADIANT), TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getNormalRotationZ(RADIANT), parsed.getNormalRotationZ(RADIANT), TOLERANCE);
		BOOST_REQUIRE_EQUAL(original.getPolygon(MILLIMETER).size(), parsed.getPolygon(MILLIMETER).size());
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[0].first, parsed.getPolygon(MILLIMETER)[0].first, TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[1].first, parsed.getPolygon(MILLIMETER)[1].first, TOLERANCE);
		BOOST_REQUIRE_CLOSE(original.getPolygon(MILLIMETER)[2].first, parsed.getPolygon(MILLIMETER)[2].first, TOLERANCE);

		BOOST_REQUIRE(original == parsed);

	} catch (std::exception &e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("PlaneData.xml"), PlaneData::getBaseTag());
	PlaneData parsed = extract<PlaneData>(fileContents);

	BOOST_REQUIRE_EQUAL("RegionDetector", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(253543797, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(253543712, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(PlaneData::LOCAL, parsed.getScope());
	BOOST_REQUIRE_EQUAL(3.0, parsed.getPolygon(MILLIMETER).size());
	BOOST_REQUIRE_CLOSE(0.0, parsed.getPolygon(MILLIMETER)[0].first, TOLERANCE);
	BOOST_REQUIRE_CLOSE(1.0, parsed.getPolygon(MILLIMETER)[1].first, TOLERANCE);
	BOOST_REQUIRE_CLOSE(2.0, parsed.getPolygon(MILLIMETER)[2].first, TOLERANCE);
	BOOST_REQUIRE_CLOSE(1.0, parsed.getTranslationX(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(1.0, parsed.getTranslationY(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(1.0, parsed.getTranslationZ(MILLIMETER), TOLERANCE);
	BOOST_REQUIRE_CLOSE(2.0, parsed.getNormalRotationX(RADIANT), TOLERANCE);
	BOOST_REQUIRE_CLOSE(2.0, parsed.getNormalRotationY(RADIANT), TOLERANCE);
	BOOST_REQUIRE_CLOSE(2.0, parsed.getNormalRotationZ(RADIANT), TOLERANCE);

}

BOOST_AUTO_TEST_SUITE_END()
