package de.unibi.airobots.btl.data;

import de.unibi.airobots.btl.common.ElementParser;

import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.Element;
import nu.xom.Attribute;

import java.util.Set;
import java.util.Map;
import java.util.HashMap;

/** 
 * Attributes are collections of measured values of
 * objects. Each attribute consists of a list of values
 * and a unique name. 
 */
public class AnchorAttribute extends Type {

	/** The measured values of the attribute. E.g. a color histogram. */
	private Map<String, Double> values;

	/** The name of the attribute. E.g. size. */
	private String name;

	/** Constructor. */
	public AnchorAttribute() {		
		values = new HashMap<String, Double>();
		this.name = "unkown";
	}
	
	/** Constructor. */
	public AnchorAttribute(String name) {
		values = new HashMap<String, Double>();
		this.name = name;
	}

	/** 
	 * Sets a new name to the attribute. 
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Adds a new value to this attribute or overrides
	 * an old one if the it has the same key.
	 */
	public void addValue(String key, double value) {
		values.put(key, value);
	}

	/** Tells the name of the attribute. */
	public String getName() {
		return name;
	}

	/** Tells the number of values stored in this attribute. */
	public int getNrValues() {
		return values.size();
	}

	public static String getBaseTag() {
		return "ATTRIBUTE";
	}
	 
	/** Returns all values and corresponding keys stored in this attribute. */
	public Map<String, Double> getValues() {	
		return values;
	}

	public void fillInto(Element parent) {

		parent.addAttribute(new Attribute("name", getName()));

		for (Map.Entry<String, Double> entry : values.entrySet()) {
			parent.addAttribute(new Attribute(entry.getKey(), String.valueOf(entry.getValue())));
		}

	}	

	public static void fromElement(Element element, AnchorAttribute attr) {

		int nrAttributes = element.getAttributeCount();
		for (int i = 0; i < nrAttributes; i++) {

			Attribute a = element.getAttribute(i);
			
			String localName = a.getLocalName();
			if (localName.equals("name"))
				attr.setName(a.getValue());
			else
				attr.addValue(a.getLocalName(), Double.parseDouble(a.getValue()));

		}	

	}

	// ========================================================================
	// Overwritten methods.
	// ========================================================================

	@Override 
	public boolean equals(Object obj) {

		if (!(obj instanceof AnchorAttribute)) 
			return false;

		AnchorAttribute attr = (AnchorAttribute) obj;
	
		if (!(values.keySet().equals(attr.getValues().keySet()))) 
			return false;		

		// Why can't we compare the value sets the same way as the key sets?
		for (String key : values.keySet()) {
				
			double value1 = values.get(key);
			double value2 = attr.getValues().get(key);
	
			if (value1 != value2) 
				return false;

		}

		return true;

	}

	@Override
	public String toString() {

		StringBuffer buffer = new StringBuffer();
		buffer.append("ATTRIBUTE: ");
		for (Map.Entry<String, Double> entry : values.entrySet()) {

			buffer.append(entry.getKey());
			buffer.append(": ");
			buffer.append(entry.getValue());
			buffer.append(" ");

		}

		return buffer.toString();

	}

	// ========================================================================
	// Fusion strategies.
	// ========================================================================

	/** 
	 * Fuses two attributes by copying the values of the assigned
	 * attribute attr to the object attribute.
	 */
	public void fuseCopy(AnchorAttribute attr) {
		values = attr.getValues();
	}

	/** 
	 * Fuses two attributes by computing the mean value between	
	 * same attribute values and setting them to the object 
	 * attribute.
	 */
	public void fuseMean(AnchorAttribute attr) {

		Map<String, Double> values2 = attr.getValues();

		// Check if both attributes contain the same key set.
		Set<String> keySet1 = values.keySet();
		Set<String> keySet2 = values2.keySet();
		if (!keySet1.equals(keySet2)) {
			System.err.println("Attributes can not be fused due to different attribute values.");
			return;
		}
			
		for (Map.Entry<String, Double> entry : values.entrySet()) {

			String key = entry.getKey();

			double value1 = entry.getValue();
			double value2 = values2.get(key);

			double newValue = (value1 + value2) / 2;
					
			values.put(key, newValue);

		}

	}

}
