package de.unibi.airobots.btl.data;

import de.unibi.airobots.btl.common.ElementParser;
import nu.xom.Attribute;
import nu.xom.Element;

/**
 * Representation of camera targets.
 * 
 * @author jwienke
 */
public class CameraTargetData extends Type {

	private double distance = 0;
	private double height = 0;
	private double azimuth = 0;

	private double zoom;

	/**
	 * Constructor.
	 */
	public CameraTargetData() {
		super();
	}

	/**
	 * Constructor.
	 */
	public CameraTargetData(double distance, double height, double azimuth) {
		super();
		this.distance = distance;
		this.height = height;
		this.azimuth = azimuth;
	}

	/**
	 * Returns the distance of the target to focus in laser coordinates (mm).
	 * 
	 * @return distance to focus in laser coordinates
	 */
	public double getDistance() {
		return distance;
	}

	/**
	 * Sets the distance of the object to focus in laser coordinates (mm).
	 * 
	 * @param distance
	 *            distance to focus in mm
	 */
	public void setDistance(double distance) {
		this.distance = distance;
	}

	/**
	 * Returns the height of the object to focus in mm above floor.
	 * 
	 * @return height of the object to focus in mm
	 */
	public double getHeight() {
		return height;
	}

	/**
	 * Sets the height of the object to focus in mm above floor.
	 * 
	 * @param height
	 *            height to focus in mm above floor
	 */
	public void setHeight(double height) {
		this.height = height;
	}

	/**
	 * Returns the azimuth angle of the object to focus in rad.
	 * 
	 * @return azimuth angle to focus (0 = straight)
	 */
	public double getAzimuth() {
		return azimuth;
	}

	/**
	 * Sets the azimuth angle of the object to focus in rad.
	 * 
	 * @param azimuth
	 *            azimuth angle to focus in rad (0 = straight)
	 */
	public void setAzimuth(double azimuth) {
		this.azimuth = azimuth;
	}

	/**
	 * Returns the zoom level of the camera as percent of the total zoom.
	 * 
	 * @return zoom [0..1]
	 */
	public double getZoom() {
		assert (zoom >= 0 && zoom <= 1) : "Zoom must be a percent value [0..1].";
		return zoom;
	}

	/**
	 * Sets the zoom as percent of the total zoom of the camera.
	 * 
	 * @param zoom
	 *            desired zoom [0..1]
	 * @throws IllegalArgumentException
	 *             illegal zoom value
	 */
	public void setZoom(double zoom) throws IllegalArgumentException {
		if (zoom < 0 || zoom > 1) {
			throw new IllegalArgumentException(
					"Zoom must be a value in [0..1] but got " + zoom + ".");
		}
		this.zoom = zoom;
	}

	public static String getBaseTag() {
		return "CAMERATARGET";
	}

	public static void fromElement(Element element, CameraTargetData type) {

		Type.fromElement(element, type);

		Element cylindricElement = element.getFirstChildElement("CYLINDRIC");
		type.setDistance(ElementParser.getDoubleAttributeValue(
				cylindricElement, "dist"));
		type.setHeight(ElementParser.getDoubleAttributeValue(cylindricElement,
				"height"));

		Element azimuthElement = element.getFirstChildElement("AZIMUTH");
		type.setAzimuth(ElementParser.getDoubleAttributeValue(azimuthElement,
				"rad"));

		Element zoomElement = element.getFirstChildElement("ZOOM");
		type
				.setZoom(ElementParser.getDoubleAttributeValue(zoomElement,
						"qual"));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);

		Element cylindricElement = new Element("CYLINDRIC");
		parent.appendChild(cylindricElement);
		cylindricElement.addAttribute(new Attribute("dist", String
				.valueOf(getDistance())));
		cylindricElement.addAttribute(new Attribute("height", String
				.valueOf(getHeight())));

		Element azimuthElement = new Element("AZIMUTH");
		parent.appendChild(azimuthElement);
		azimuthElement.addAttribute(new Attribute("rad", String
				.valueOf(getAzimuth())));

		Element zoomElement = new Element("ZOOM");
		parent.appendChild(zoomElement);
		zoomElement.addAttribute(new Attribute("qual", String
				.valueOf(getZoom())));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "CameraTargetData [distance = " + distance + ", height = "
				+ height + ", azimuth = " + azimuth + ", zoom = " + zoom + "]";
	}

}
