package de.unibi.airobots.btl.data;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import de.unibi.airobots.btl.common.ElementParser;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

/**
 * List of all face identifications found.
 * 
 * @author jwienke
 */
public class FaceIdentificationList extends Type implements
		Iterable<FaceIdentificationList.FaceIdentification> {

	/**
	 * One single result of the face identification.
	 * 
	 * @author jwienke
	 */
	public static class FaceIdentification {

		private int regionX;
		private int regionY;
		private int regionWidth;
		private int regionHeight;
		private Integer classId = null;

		FaceIdentification() {
		}

		public int getRegionX() {
			return regionX;
		}

		public int getRegionY() {
			return regionY;
		}

		public int getRegionWidth() {
			return regionWidth;
		}

		public int getRegionHeight() {
			return regionHeight;
		}

		public Integer getClassId() {
			return classId;
		}

		void setRegionX(int regionX) {
			this.regionX = regionX;
		}

		void setRegionY(int regionY) {
			this.regionY = regionY;
		}

		void setRegionWidth(int regionWidth) {
			this.regionWidth = regionWidth;
		}

		void setRegionHeight(int regionHeight) {
			this.regionHeight = regionHeight;
		}

		void setClassId(Integer classId) {
			this.classId = classId;
		}

		public boolean isIdentified() {
			return classId != null;
		}

		/**
		 * {@inheritsDoc}
		 */
		@Override
		public String toString() {
			StringBuilder strb = new StringBuilder();
			strb.append("#FACEIDENTIFICATION# ");
			strb.append("identified: " + isIdentified() + "; ");
			strb.append("classId: " + getClassId() + "; ");
			strb.append("regionX: " + regionX + "; ");
			strb.append("regionY: " + regionY + "; ");
			strb.append("width: " + regionWidth + "; ");
			strb.append("height: " + regionHeight + "; ");

			return strb.toString();
		}

	}

	private Set<FaceIdentification> identifications = new HashSet<FaceIdentification>();
	
	public FaceIdentificationList() {
		// The provided data don't have a timestamp and generator
		setLegacyParsing(true);
	}
	
	private void addIdentification(FaceIdentification identification) {
		identifications.add(identification);
	}
	
	public static String getBaseTag() {
		return "FACES";
	}

	public static void fromElement(Element element, FaceIdentificationList list) {

		Type.fromElement(element, list);

		// parse all faces in the document
		Nodes nodes = element.query("FACE");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);

			// parse the face
			if (node instanceof Element) {
				Element elem = (Element) node;

				FaceIdentification ident = new FaceIdentification();

				Nodes children = elem.query("*");
				for (int j = 0; j < children.size(); j++) {

					if (!(children.get(j) instanceof Element)) {
						continue;
					}
					Element child = (Element) children.get(j);

					if (child.getLocalName().equals("REGION")) {

						Nodes subchildren = child.query("*");
						for (int k = 0; k < subchildren.size(); k++) {

							if (!(subchildren.get(k) instanceof Element)) {
								continue;
							}

							Element subchild = (Element) subchildren.get(k);
							if (subchild.getLocalName().equals("RECTANGLE")) {
								ident.setRegionX(ElementParser
										.getIntAttributeValue(subchild, "x"));
								ident.setRegionY(ElementParser
										.getIntAttributeValue(subchild, "y"));
								ident.setRegionWidth(ElementParser
										.getIntAttributeValue(subchild, "w"));
								ident.setRegionHeight(ElementParser
										.getIntAttributeValue(subchild, "h"));
							}

						}

					} else if (child.getLocalName().equals("CLASS")) {

						String classId = ElementParser.getAttributeValue(child,
								"class-id");
						try {
							ident.setClassId(Integer.parseInt(classId));
						} catch (NumberFormatException e) {
							ident.setClassId(null);
						}

					}

				}

				list.addIdentification(ident);

			}
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		throw new UnsupportedOperationException();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Iterator<FaceIdentification> iterator() {
		return identifications.iterator();
	}

	/**
	 * Returns the number of identification results in this list.
	 * 
	 * @return size >= 0
	 */
	public int size() {
		return identifications.size();
	}

}
