package de.unibi.airobots.btl.data;

import java.util.HashSet;
import java.util.Set;

import de.unibi.airobots.btl.common.Polygon;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

/**
 * Results of the object recognition. This class is meat so define the location
 * of the object in the detector's camera image! The given polygon describes the
 * objects's location in pixel coordinates! If you want to define an object in
 * world coordinates use {@link ObjectPositionData}.
 * 
 * @author lziegler
 */
public class ObjectLocationData extends ObjectData {

	protected final static String SENSORTYPE_ATTRIBUTE = "SENSORTYPE";

	public static class Hypothesis extends ObjectData.Hypothesis {

		/**
		 * Position of the object in the sensor's image.
		 */
		private Polygon polygon;

		/**
		 * Description of the hardware, that recorded the location.
		 */
		private String sensorType;

		public Polygon getPolygon() {
			return polygon;
		}

		public void setPolygon(Polygon polygon) {
			this.polygon = polygon;
		}

		public String getSensorType() {
			return sensorType;
		}

		public void setSensorType(String sensorType) {
			this.sensorType = sensorType;
		}

		/**
		 * Getter for the xml base tag used for this (de-)serialization.
		 * 
		 * @return xml base tag
		 */
		public static String getBaseTag() {
			return "HYPOTHESIS";
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void fillInto(Element parent) {

			super.fillInto(parent);

			// polygon
			Element polygonElem = new Element(Polygon.getBaseTag());
			polygon.fillInto(polygonElem);

			// sensor type
			Element sensorElem = new Element(SENSORTYPE_ATTRIBUTE);
			sensorElem.appendChild(sensorType);

			parent.appendChild(polygonElem);
			parent.appendChild(sensorElem);
		}

		/**
		 * Fills an {@link ObjectLocationData.Hypothesis} object from a given
		 * XOM {@link Element}.
		 * 
		 * @param hypothesisElement
		 *            The XOM {@link Element} to fill an object from.
		 * @param type
		 *            The {@link ObjectLocationData.Hypothesis} object to fill
		 *            with all the information given by the {@link Element}
		 *            object.
		 */
		public static void fromElement(Element hypothesisElement,
				Hypothesis type) {

			ObjectData.Hypothesis.fromElement(hypothesisElement, type);

			type.setPolygon(Polygon.fromElement(hypothesisElement
					.getFirstChildElement(Polygon.getBaseTag())));
			type.setSensorType(hypothesisElement.getFirstChildElement(
					SENSORTYPE_ATTRIBUTE).getValue());
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public String toString() {
			return "[HYPOTHESIS timestamp: " + getTimestamp()
					+ " reliability:  " + getReliability() + " sensor type: "
					+ sensorType + " Polygon: " + polygon + "]";

		}
	}

	/**
	 * Default constructor.
	 */
	public ObjectLocationData() {
		setLegacyParsing(true);
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "OBJECTLOCATION";
	}

	public Set<Hypothesis> getLocationHypotheses() {

		HashSet<Hypothesis> set = new HashSet<Hypothesis>();
		for (ObjectData.Hypothesis h : hypotheses) {
			if (h instanceof Hypothesis) {
				set.add((Hypothesis) h);
			}
		}
		return set;
	}

	/**
	 * @deprecated This method is not valid for subclasses of {@link ObjectData}
	 *             . Use {@link #getLocationHypotheses()} instead.
	 */
	@Deprecated
	public Set<ObjectData.Hypothesis> getHypotheses() {
		return super.getHypotheses();
	}

	/**
	 * Fills an {@link ObjectLocationData} object from a given XOM
	 * {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectLocationData} object to fill with all the
	 *            information given by the {@link Element} object.
	 */
	public static void fromElement(Element objectElement,
			ObjectLocationData type) {

		try {

			ObjectData.fromElement(objectElement, type);

			// clean up
			type.clearHypotheses();

			// check hypotheses
			Nodes nodes = objectElement.query(Hypothesis.getBaseTag());
			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);
				if (node instanceof Element) {

					Hypothesis hyp = new Hypothesis();
					Hypothesis.fromElement((Element) node, hyp);
					type.addHypothesis(hyp);
				}
			}

		} catch (NullPointerException ex) {
			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute "
					+ "in document.", ex);
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return super.toString().replace("[OBJECT", "[OBJECTLOCATION");
	}
}
