package de.unibi.airobots.btl.data;

import java.util.HashSet;
import java.util.Set;

import de.unibi.airobots.btl.common.PrecisePolygon;
import de.unibi.airobots.btl.units.LengthUnit;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParsingException;

/**
 * Results of the object recognition. This class is meant so define the position of the object in
 * the world! The given polygon describes the objects's position in world coordinates! If you want
 * to define the location in image coordinates use {@link ObjectLocationData}.
 * 
 * @author lziegler
 */
public class ObjectPositionData extends ObjectData {

	private static final double DEFAULT_EDGE_LENGTH = 0.1;
	private static final double DEFAULT_RELIABILITY = 1.0;

	public static class Hypothesis extends ObjectData.Hypothesis {

		/**
		 * Position of the object in the sensor's image.
		 */
		private PrecisePolygon polygon;

		private String reference = "";

		private String coordinateKind = "";

		public PrecisePolygon getPolygon() {
			return polygon;
		}

		public void setPolygon(PrecisePolygon polygon) {
			this.polygon = polygon;
		}

		public String getReference() {
			return reference;
		}

		public void setReference(String reference) {
			this.reference = reference;
		}

		public String getCoordinateKind() {
			return coordinateKind;
		}

		public void setCoordinateKind(String coordinateKind) {
			this.coordinateKind = coordinateKind;
		}

		/**
		 * Getter for the xml base tag used for this (de-)serialization.
		 * 
		 * @return xml base tag
		 */
		public static String getBaseTag() {
			return "HYPOTHESIS";
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public void fillInto(Element parent) {

			super.fillInto(parent);

			// polygon
			Element polygonElem = new Element(PrecisePolygon.getBaseTag());
			polygon.fillInto(polygonElem);
			polygonElem.addAttribute(new Attribute("kind", coordinateKind));
			polygonElem.addAttribute(new Attribute("ref", reference));

			parent.appendChild(polygonElem);
		}

		/**
		 * Fills an {@link ObjectPositionData.Hypothesis} object from a given XOM {@link Element}.
		 * 
		 * @param hypothesisElement
		 *            The XOM {@link Element} to fill an object from.
		 * @param type
		 *            The {@link ObjectPositionData.Hypothesis} object to fill with all the
		 *            information given by the {@link Element} object.
		 * @throws ParsingException
		 */
		public static void fromElement(Element hypothesisElement, Hypothesis type)
				throws ParsingException {

			ObjectData.Hypothesis.fromElement(hypothesisElement, type);

			Element polygonElem = hypothesisElement.getFirstChildElement(PrecisePolygon
					.getBaseTag());

			type.setPolygon(PrecisePolygon.fromElement(polygonElem));
			type.setCoordinateKind(polygonElem.getAttributeValue("kind"));
			type.setReference(polygonElem.getAttributeValue("ref"));
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
		public String toString() {
			return "[HYPOTHESIS timestamp: " + getTimestamp() + " reliability:  "
					+ getReliability() + " Polygon: " + polygon + "]";

		}
	}

	/**
	 * Default constructor.
	 */
	public ObjectPositionData() {
		setLegacyParsing(true);
	}

	public Set<Hypothesis> getLocationHypotheses() {

		HashSet<Hypothesis> set = new HashSet<Hypothesis>();
		for (ObjectData.Hypothesis h : hypotheses) {
			if (h instanceof Hypothesis) {
				set.add((Hypothesis) h);
			}
		}
		return set;
	}

	/**
	 * @deprecated This method is not valid for subclasses of {@link ObjectData} . Use
	 *             {@link #getLocationHypotheses()} instead.
	 */
	@Deprecated
	public Set<ObjectData.Hypothesis> getHypotheses() {
		return super.getHypotheses();
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "OBJECTPOSITION";
	}

	/**
	 * Fills an {@link ObjectPositionData} object from a given XOM {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectPositionData} object to fill with all the information given by
	 *            the {@link Element} object.
	 * @throws ParsingException
	 */
	public static void fromElement(Element objectElement, ObjectPositionData type)
			throws ParsingException {

		try {

			ObjectData.fromElement(objectElement, type);

			// clean up
			type.clearHypotheses();

			// check hypotheses
			Nodes nodes = objectElement.query(Hypothesis.getBaseTag());
			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);
				if (node instanceof Element) {

					Hypothesis hyp = new Hypothesis();
					Hypothesis.fromElement((Element) node, hyp);
					type.addHypothesis(hyp);
				}
			}

		} catch (NullPointerException ex) {
			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute " + "in document.", ex);
		}
	}

	public static ObjectPositionData fromPosition(PositionData position, String label) {
		return fromPosition(position, label, DEFAULT_EDGE_LENGTH, LengthUnit.METER,
				DEFAULT_RELIABILITY);
	}

	public static ObjectPositionData fromPosition(PositionData position, String label,
			double reliability) {
		return fromPosition(position, label, DEFAULT_EDGE_LENGTH, LengthUnit.METER, reliability);
	}

	public static ObjectPositionData fromPosition(PositionData position, String label,
			double edgeLength, LengthUnit unit) {
		return fromPosition(position, label, edgeLength, unit, DEFAULT_RELIABILITY);
	}

	public static ObjectPositionData fromPosition(PositionData position, String label,
			double edgeLength, LengthUnit unit, double reliability) {
		double half = edgeLength / 2.0;
		PrecisePolygon poly = new PrecisePolygon();
		poly.addPoint(position.getX() - half, position.getY() - half, unit);
		poly.addPoint(position.getX() - half, position.getY() + half, unit);
		poly.addPoint(position.getX() + half, position.getY() + half, unit);
		poly.addPoint(position.getX() + half, position.getY() - half, unit);
		Hypothesis hyp = new Hypothesis();
		hyp.setPolygon(poly);
		hyp.setReliability(reliability);
		ObjectPositionData obj = new ObjectPositionData();
		obj.addHypothesis(hyp);
		obj.setClassLabel(label);
		return obj;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return super.toString().replace("[OBJECT", "[OBJECTPOSITION");
	}
}
