package de.unibi.airobots.btl.data;

import nu.xom.Element;
import nu.xom.Nodes;
import nu.xom.ParsingException;

/**
 *
 * @author fsiepman
 */
public class RobotState extends Type {

    private static final String CURRENT_STATE_ELEMENT_NAME = "CURRENTSTATE";
    private static final String PREVIOUS_STATE_ELEMENT_NAME = "PREVIOUSSTATE";
    private static final String STATE_TARGET_ELEMENT_NAME = "STATETARGET";
    
    public static enum RobotStates {

        /**
         * The robot is driving to a position
         */
        DRIVE_TO,
        /**
         * The robot shall speak to an interaction partner
         */
        TALK_TO,
        /**
         * The robot shall return to the HOME position, normally (0,0)
         */
        GO_HOME,
        /**
         * The robot is waiting for command somewhere
         */
        WAITING
    }
    private RobotStates previousState;
    private RobotStates currentState;
    private String targetName;
    

    public RobotState() {
    }

    /**
     * Returns the current state of the robot.
     *
     * @return RobotState current state of the robot
     */
    public RobotStates getCurrenState() {
        return currentState;
    }

    /**
     * Returns the previous state of the robot.
     *
     * @return RobotState previous state of the robot
     */
    public RobotStates getPreviousState() {
        return previousState;
    }

    /**
     * Set the current robot state
     *
     * @param currenState current state of the robot
     */
    public void setCurrenState(RobotStates currenState) {
        this.currentState = currenState;
    }

    /**
     * Set previous robot state
     * 
     * @param previousState previous state of the robot
     */
    public void setPreviousState(RobotStates previousState) {
        this.previousState = previousState;
    }

    public void setTargetName(String targetName) {
        this.targetName = targetName;
    }

    public String getTargetName() {
        return targetName;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals(Object obj) {

        if (!(obj instanceof RobotState)) {
            return false;
        }

        RobotState other = (RobotState) obj;

        return super.equals(obj) && (other.currentState.equals(currentState)
                && other.previousState.equals(previousState)
                && other.targetName.equals(targetName));

    }

    public static String getBaseTag() {
        return "ROBOTSTATE";
    }

    /**
     *
     */
    public static void fromElement(Element element, RobotState robotState) throws ParsingException {

        Type.fromElement(element, robotState);

        // Getting current state from element
        Element stateElement = element.getFirstChildElement(CURRENT_STATE_ELEMENT_NAME);

        // Getting previous state from element
        Element previousStateElement = element.getFirstChildElement(PREVIOUS_STATE_ELEMENT_NAME);

        // Getting state target
        Element stateTarget = element.getFirstChildElement(STATE_TARGET_ELEMENT_NAME);

        if (stateElement == null) {
            throw new ParsingException("Unable to find element with name "
                    + CURRENT_STATE_ELEMENT_NAME + ".");
        } else if (previousStateElement == null) {
            throw new ParsingException("Unable to find element with name "
                    + PREVIOUS_STATE_ELEMENT_NAME + ".");
        } else if (stateTarget == null) {
            throw new ParsingException("Unable to find element with name "
                    + STATE_TARGET_ELEMENT_NAME + ".");
        }

        String stateName = stateElement.getValue();
        if ((stateName == null) || stateName.trim().isEmpty()) {
            throw new ParsingException("No state found in element with name "
                    + CURRENT_STATE_ELEMENT_NAME + ".");
        }

        String previousStateName = previousStateElement.getValue();
        if ((stateName == null) || stateName.trim().isEmpty()) {
            throw new ParsingException("No state found in element with name "
                    + PREVIOUS_STATE_ELEMENT_NAME + ".");
        }

        String targetName = stateTarget.getValue();
        if ((targetName == null) || targetName.trim().isEmpty()) {
            throw new ParsingException("No state found in element with name "
                    + STATE_TARGET_ELEMENT_NAME + ".");
        }

        try {
            RobotStates current = RobotStates.valueOf(stateName);
            RobotStates previous = RobotStates.valueOf(previousStateName);
            robotState.setCurrenState(current);
            robotState.setPreviousState(previous);
            robotState.setTargetName(targetName);

        } catch (IllegalArgumentException e) {
            throw new ParsingException("Unable to extract valid RobotState.");
        } catch (NullPointerException e) {
            throw new ParsingException("Unable to extract valid RobotState. Don't you dare creating NPE's...");
        }

    }

	@Override
	public void sanitizeElement(Element parent) {
		super.sanitizeElement(parent);
		
		Nodes nodes = parent.query("CURRENTSTATE");
		for (int i = 0; i < nodes.size(); i++) {
			parent.removeChild(nodes.get(i));
		}
		
		nodes = parent.query("PREVIOUSSTATE");
		for (int i = 0; i < nodes.size(); i++) {
			parent.removeChild(nodes.get(i));
		}
		
		nodes = parent.query("STATETARGET");
		for (int i = 0; i < nodes.size(); i++) {
			parent.removeChild(nodes.get(i));
		}
		
	}
    
    /**
     * {@inheritDoc}
     */
    @Override
    public void fillInto(Element parent) {
    	
    	sanitizeElement(parent);
    	
        super.fillInto(parent);

        Element stateElement = new Element(CURRENT_STATE_ELEMENT_NAME);
        Element previousElement = new Element(PREVIOUS_STATE_ELEMENT_NAME);
        Element targetElement = new Element(STATE_TARGET_ELEMENT_NAME);
        stateElement.appendChild(currentState.name());
        previousElement.appendChild(previousState.name());
        targetElement.appendChild(targetName);
        parent.appendChild(stateElement);
        parent.appendChild(previousElement);
        parent.appendChild(targetElement);

    }
}
