package de.unibi.airobots.btl.data;

import org.apache.log4j.Logger;

import de.unibi.airobots.btl.common.ElementParser;
import de.unibi.airobots.btl.common.MicroTimestamp;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParsingException;

/**
 * This class is used to store data from the speed sensors. It holds the
 * translational Speed[m/s], rotational Speed[rad/s] and the time of these
 * sensor values as a timestamp.
 * 
 * @author jwienke
 * @author unknown
 */
public class SpeedData extends Type implements Ticked {

	private static Logger logger = Logger.getLogger(SpeedData.class);
	
	private double speedTranslation;
	private double speedRotation;
	private MicroTimestamp readTime = new MicroTimestamp();

	/**
	 * @return the translational Speed[m/s]
	 */
	public double getSpeedTranslation() {
		return speedTranslation;
	}

	/**
	 * @return the rotational Speed[rad/s]
	 */
	public double getSpeedRotation() {
		return speedRotation;
	}

	/**
	 * Sets the translational speed.
	 * 
	 * @param speedTranslation
	 *            translational speed in [m/s]
	 */
	public void setSpeedTranslation(double speedTranslation) {
		this.speedTranslation = speedTranslation;
	}

	/**
	 * Sets the rotational speed.
	 * 
	 * @param speedRotation
	 *            rotational speed [rad/s]
	 */
	public void setSpeedRotation(double speedRotation) {
		this.speedRotation = speedRotation;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append("#SPEED# ");
		strb.append("timestamp: " + getTimestamp() + "; ");
		strb.append("translation: " + speedTranslation + "; ");
		strb.append("rotation: " + speedRotation + ";");
		return strb.toString();
	}

	public static String getBaseTag() {
		return "SPEEDS";
	}

	public static void fromElement(Element element, SpeedData type)
			throws ParsingException {
		
		logger.debug("fromElement called");

		Type.fromElement(element, type);

		Nodes nodes = element.query("*");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (elem.getLocalName().equals("ROTATIONSPEED")) {
					type.setSpeedRotation(ElementParser
							.getDoubleAttributeValue(elem, "quant"));
				} else if (elem.getLocalName().equals("TRANSLATIONSPEED")) {
					type.setSpeedTranslation(ElementParser
							.getDoubleAttributeValue(elem, "quant"));
				} else if (elem.getLocalName().equals(
						MicroTimestamp.getBaseTag())) {
					MicroTimestamp readTime = new MicroTimestamp();
					MicroTimestamp.fromElement(elem, readTime);
					type.setReadTime(readTime);
				}
			}
		}

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		Element readTimeElement = new Element(MicroTimestamp.getBaseTag());
		parent.appendChild(readTimeElement);
		readTime.fillInto(readTimeElement);

		Element rotElement = new Element("ROTATIONSPEED");
		parent.appendChild(rotElement);
		rotElement.addAttribute(new Attribute("quant", String
				.valueOf(getSpeedRotation())));

		Element transElement = new Element("TRANSLATIONSPEED");
		parent.appendChild(transElement);
		transElement.addAttribute(new Attribute("quant", String
				.valueOf(getSpeedTranslation())));

	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public MicroTimestamp getReadTime() {
		return readTime;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeMicroSeconds() {
		return readTime.getMicroSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeSeconds() {
		return readTime.getSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setReadTime(MicroTimestamp readTime) {
		this.readTime = readTime;
	}

}
