package de.unibi.airobots.btl.data;

import java.util.ArrayList;
import java.util.List;

import nu.xom.Attribute;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.common.ElementParser;
import de.unibi.airobots.btl.common.MicroTimestamp;

/**
 * BTL type for work with the output from the sploc ES speaker localization
 * application.
 * 
 * @author dschulze
 * @author dklotz
 */
public class SplocAngleData extends Type implements Ticked {

	private static final String ANGLE_TAG = "ANGLE";

	/** A list of all angles computed from the SplocES. */
	private List<Double> angles = new ArrayList<Double>();
	/** The exact timestamp the data was read / computed. */
	private MicroTimestamp readTime = new MicroTimestamp();

	// Constants for all the XML field names
	private static final String BASE_TAG = "SPLOCANGLES";
	private static final String ANGLES_TAG = "ANGLES";
	private static final String VALUE_ATTRIBUTE = "value";

	public static String getBaseTag() {
		return BASE_TAG;
	}

	public void setAngles(List<Double> angles) {
		this.angles = angles;
	}

	public List<Double> getAngles() {
		return angles;
	}

	/** @{inheritDoc */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);
		
		// Append the exact timestamp
		Element readTimeElement = new Element(MicroTimestamp.getBaseTag());
		parent.appendChild(readTimeElement);
		readTime.fillInto(readTimeElement);

		// Append angle information.
		Element anglesElement = new Element(ANGLES_TAG);

		for (Double angleValue : angles) {
			Element angleElement = new Element(ANGLE_TAG);
			angleElement.addAttribute(new Attribute(VALUE_ATTRIBUTE, String
					.valueOf(angleValue)));
			anglesElement.appendChild(angleElement);
		}

		parent.appendChild(anglesElement);

	}

	/** @{inheritDoc */
	public static void fromElement(Element element, SplocAngleData data)
			throws ParsingException {

		Type.fromElement(element, data);

		List<Double> angles = new ArrayList<Double>();

		Element anglesElement = element.getFirstChildElement(ANGLES_TAG);
		Elements angleElements = anglesElement.getChildElements(ANGLE_TAG);

		for (int i = 0; i < angleElements.size(); i++) {
			Element angleElement = angleElements.get(i);
			angles.add(ElementParser.getDoubleAttributeValue(angleElement,
					VALUE_ATTRIBUTE));
		}

		data.setAngles(angles);

		Element readTimeElement = element.getFirstChildElement(MicroTimestamp
				.getBaseTag());
		MicroTimestamp readTime = new MicroTimestamp();
		MicroTimestamp.fromElement(readTimeElement, readTime);
		data.setReadTime(readTime);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public MicroTimestamp getReadTime() {
		return readTime;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeMicroSeconds() {
		return readTime.getMicroSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public long getReadTimeSeconds() {
		return readTime.getSeconds();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setReadTime(MicroTimestamp readTime) {
		this.readTime = readTime;
	}

}
