package de.unibi.airobots.btl.data.asu;

import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;

/**
 * Provides access to all stored frames.
 * 
 * @author malinke
 */
public class FrameDB implements Iterable<Entry<String, FrameEntry>> {

	private Map<String, FrameEntry> frameMap = new TreeMap<String, FrameEntry>();

	/**
	 * Loads the complete database.
	 * 
	 * @param fileName
	 *            of database file
	 */
	public FrameDB(final String fileName) {
		loadDatabase(fileName);
	}

	/**
	 * Returns the frame entry of the given name.
	 * 
	 * @param frameName
	 *            name of the wanted frame
	 * @return frame's entry
	 */
	public final FrameEntry getFrame(final String frameName) {
		return this.frameMap.get(frameName);
	}

	/**
	 * Returns the names of all frames.
	 * 
	 * @return names of all frames
	 */
	public final String[] getFrameNames() {
		String[] frameNames = new String[this.frameMap.size()];
		return this.frameMap.keySet().toArray(frameNames);
	}

	/**
	 * {@inheritDoc}
	 */
	public final Iterator<Entry<String, FrameEntry>> iterator() {
		return this.frameMap.entrySet().iterator();
	}

	private void loadDatabase(final String fileName) {
		Builder b = new Builder();
		try {
			Document dbDoc = b.build(fileName);
			Elements entries = dbDoc.getRootElement().getChildElements("frame");
			createEntries(entries);
			connectFrames(entries);
		} catch (ParsingException ex) {
			throw new IllegalArgumentException("Could not parse db file: " + ex);
		} catch (IOException ex) {
			throw new IllegalArgumentException("Could not read db file: " + ex);
		}

	}

	private void createEntries(final Elements entries) {
		for (int i = 0; i < entries.size(); i++) {
			Element entry = entries.get(i);
			String name = entry.getAttributeValue("name");
			this.frameMap.put(name, new FrameEntry(name));
		}
	}

	private void connectFrames(final Elements entries) {
		for (int i = 0; i < entries.size(); i++) {
			Element entry = entries.get(i);
			String name = entry.getAttributeValue("name");
			FrameEntry frame = this.frameMap.get(name);
			connectTopFrame(frame, entry);
			connectSlots(frame, entry);
		}
	}

	private void connectTopFrame(final FrameEntry frame, final Element entry) {
		if (entry.getAttribute("topFrame") == null) {
			return;
		}
		String topFrameName = entry.getAttributeValue("topFrame");
		frame.setTopFrame(this.frameMap.get(topFrameName));
	}

	private void connectSlots(final FrameEntry frame, final Element entry) {
		Elements slots = entry.getChildElements("slot");
		for (int i = 0; i < slots.size(); i++) {
			String slotName = slots.get(i).getAttributeValue("name");
			if (slotName.isEmpty()) {
				/* ignore empty slots */
				continue;
			}
			frame.addSlot(this.frameMap.get(slotName));
		}
	}

}
