package de.unibi.airobots.btl.data.asu;

import java.util.ArrayList;
import java.util.List;

/**
 * Represents a node in a semantical word net. It combines a word with a frame
 * name (situated semantic unit, SSU).
 * 
 * @author malinke
 */
public class WordNode {

	private final String word;

	private final String frameName;

	private final List<WordNode> leaves;

	private String slotName;

	/**
	 * Creates a new node.
	 * 
	 * @param word
	 *            looked up in the lexicon
	 * @param frame
	 *            one meaning of the word
	 */
	public WordNode(final String word, final String frameName) {
		this.word = word;
		this.frameName = frameName;
		this.leaves = new ArrayList<WordNode>();
	}

	/**
	 * Simply returns the word of this node.
	 * 
	 * @return looked up in the lexicon
	 */
	public final String getWord() {
		return this.word;
	}

	/**
	 * Adds a leave to the list of leaves to build a tree structure.
	 * 
	 * @param leave
	 *            to add
	 */
	public final void addLeave(WordNode leave) {
		this.leaves.add(leave);
	}

	/**
	 * Provides the list of nodes connected to this.
	 * 
	 * @return direct connected nodes of the next level
	 */
	public final List<WordNode> getLeaves() {
		return this.leaves;
	}

	/**
	 * Provides the name of the frame of this node.
	 * 
	 * @return name of the frame
	 */
	public final String getFrameName() {
		return this.frameName;
	}

	/**
	 * Provides the name of the slot, if set. If this node is the leave of
	 * another node, the slot is the required frame name of the slot of the
	 * parent node.
	 * 
	 * @return the slot name, if this node has a parent, null otherwise
	 */
	public final String getSlotName() {
		return this.slotName;
	}

	/**
	 * Sets the slot name. Has to be called, if this node is connected as leave.
	 * 
	 * @param name
	 *            The name of the slot, where this node fits into.
	 */
	public final void setSlotName(String name) {
		this.slotName = name;
	}

}
